import { devServer as devServer$1 } from '@cypress/webpack-dev-server';
import * as fs from 'fs';
import * as path from 'path';
import { AngularWebpackPlugin } from '@ngtools/webpack';
import * as linkerPlugin from '@angular/compiler-cli/linker/babel';
import { dirSync } from 'tmp';

const generateTsConfigContent = (specPattern, projectRoot) => {
    const getFilePath = (fileName) => path.join(projectRoot, fileName);
    const getIncludePaths = () => {
        const d = getFilePath('src/**/*.d.ts');
        if (Array.isArray(specPattern)) {
            return [d, ...specPattern.map(sp => getFilePath(sp))];
        }
        if (typeof specPattern === 'string') {
            return [d, getFilePath(specPattern)];
        }
        return [d];
    };
    return `
{
  "extends": "${getFilePath('tsconfig.json')}",
  "compilerOptions": {
    "outDir": "${getFilePath('out-tsc/cy')}",
    "types": ["${getFilePath('node_modules/cypress')}"],
    "allowSyntheticDefaultImports": true
  },
  "include": [${getIncludePaths().map(x => `"${x}"`)}]
}
`;
};
const generateTsConfig = (specPattern, projectRoot, tempDir) => {
    const tsconfigContent = generateTsConfigContent(specPattern, projectRoot);
    fs.writeFileSync(`${tempDir}/tsconfig.cy.json`, tsconfigContent);
};

const getWebpackConfig = (tmpDir) => ({
    mode: 'development',
    module: {
        rules: [
            {
                test: /\.[jt]sx?$/,
                loader: '@ngtools/webpack',
            },
            {
                test: /(\.scss|\.sass)$/,
                use: ['raw-loader', 'sass-loader'],
            },
            {
                test: /\.m?js$/,
                use: {
                    loader: 'babel-loader',
                    options: {
                        plugins: [linkerPlugin.default],
                        compact: false,
                        cacheDirectory: true
                    }
                }
            }
        ],
    },
    resolve: {
        extensions: ['.ts', '.js'],
    },
    plugins: [
        new AngularWebpackPlugin({
            tsconfig: `${path.join(tmpDir, "/tsconfig.cy.json")}`
        }),
    ],
});

function devServer(cypressDevServerConfig) {
    const { config } = cypressDevServerConfig;
    const { specPattern, projectRoot } = config;
    // This creates a temporary directory used to store a tsconfig.cy.json file needed to construct the AngularWebpackPlugin
    const { name: tempDir } = dirSync();
    // This generates the tsconfig.cy.json file in the temporary directory from above
    generateTsConfig(specPattern, projectRoot, tempDir);
    return devServer$1(cypressDevServerConfig, { webpackConfig: getWebpackConfig(tempDir) });
}

/*
 * Public API Surface of angular-dev-server
 */

/**
 * Generated bundle index. Do not edit.
 */

export { devServer };
//# sourceMappingURL=cypress-angular-dev-server.js.map
